#!/bin/bash

# Copyright (C) 2015 ARM Ltd.  All rights reserved.

set -e

AOSP_URL="https://android.googlesource.com/platform/manifest/"
AOSP_REV="android-5.0.0_r4.0.1"
AOSP_BIN_URL="https://dl.google.com/dl/android/aosp"
AOSP_FW_CODE="LRX21P"
AOSP_FACTORY="mantaray-lrx21p"
AOSP_FACTORY_TGZ="$AOSP_FACTORY""-factory-ad2499ea.tgz"
KERNEL_URL="https://android.googlesource.com/kernel/exynos"
KERNEL_BRANCH="android-exynos-manta-3.4-lollipop-release"
KERNEL_REV="f29944582ffc979a67ff597f7573a600238c9578"
MALI_VERSION="r5p0-06rel0"
FULL_PACKAGE="nexus10-mali-$MALI_VERSION"

print_usage_exit()
{
    arg_ret="${1-1}"

    echo "ARM Mali Nexus 10 AOSP developer tool.

This script can be used to build the Android Open Source Project
(AOSP) from source and download the the latest Mali GPU user-side
binary drivers to produce an engineering build for development
purposes on the Google Nexus 10 device (a.k.a Manta).  For more
information, please visit the following page:

  http://community.arm.com/docs/DOC-10124


Usage:

  ./nexus10-setup.sh COMMAND [OPTIONS...]

Each command may take an arbitrary number of options and some commands
rely on other commands to have been run first or for some manual
intervention (for example, getting some source code and toolchains
before building).  Each command is detailed in the list below in the
order that they should typically be run.


List of commands:

  do_everything
    Run all the commands listed below to start from scratch and
    produce a full Android binary ready to be flashed on a Nexus 10
    device.

  get_aosp [URL] [REV]
    Download the AOSP source code in the \"aosp\" directory.  The URL
    and REV options define an alternative AOSP manifest Git repository
    URL and the Git revision to use respectively.  Typically, the URL
    can be set to use a local mirror and the REV to use a different
    AOSP version.

    This command will also download proprietary firmware files from
    the Google website.  Special terms and conditions apply to the use
    of these files and they will need to be accepted on the command
    line while unpacking the download archives.

  get_kernel [URL]
    Set up the kernel source code in the \"kernel\" directory.  First,
    clone the initial Linux kernel source code from the AOSP Git
    repository.  The URL optional argument can be set to use a
    different Git URL.  Then this command creates a branch and applies
    patches to use latest Mali GPU driver and enable development
    options.  It does this on a new branch, and only if that new
    branch doesn't already exists.  This command finally configures
    the kernel source tree using manta_defconfig.

  build_kernel
    Build the linux kernel using the source code contained in the
    \"kernel\" directory.

  build_gator
    If the \"gator-driver\" and \"gator-daemon\" directories are
    present, build the kernel-side Gator driver and the user-side
    Gator daemon.  This is required in order to be able to use DS-5
    Streamline with the device.  It is also possible to install Gator
    later, on a running device.

  build_aosp
    Build the AOSP source code found in the \"aosp\" directory.  This
    will also create the Android file systems including user-side Mali
    GPU binary drivers which will need to be manually downloaded
    first.  It will also create a boot.img file with the previously
    built Linux kernel from the \"kernel\" directory.  If the Gator
    driver and daemon were build then they will also be added to the
    Android file system.

  package
    Create a tarball with all the components to be flashed on a Nexus
    10 device.  This command first downloads a stock tarball from
    Google to copy the pre-built binary bootloader and the flash-all
    utility scripts into the new tarball.
"

    exit $arg_ret
}

cmd="$1"
[ -z "$cmd" ] && print_usage_exit
shift

[ -z "$CROSS_COMPILE" ] && export CROSS_COMPILE=\
$PWD/aosp/prebuilts/gcc/linux-x86/arm/arm-eabi-4.8/bin/arm-eabi-
export ARCH=arm

# -----------------------------------------------------------------------------
# Utility functions

jopt()
{
    echo "-j"$(grep -c processor /proc/cpuinfo)
}

# -----------------------------------------------------------------------------
# Functions to run each command

cmd_get_aosp()
{
    arg_url="${1-$AOSP_URL}"
    arg_rev="${2-$AOSP_REV}"

    case "$AOSP_REV" in
        android-5.0.0_r4.0.1)
            sx="android-5.0_1"
            ;;
        *)
            echo "Unsupported Android version"
            exit 1
    esac

    mali_name=mali-t60x_"$MALI_VERSION"_"$sx"
    mali_archive="$mali_name".tar.gz
    [ -e "$mali_archive" ] || {
        echo "Mali driver archive not found: $mali_archive"
        exit 1
    }

    mkdir -p aosp
    cd aosp
    repo init -u "$arg_url" -b "$arg_rev"

    fw_code=$([ -e FIRMWARE_CODE ] && cat FIRMWARE_CODE || echo '')
    if [ "$fw_code" != "$AOSP_FW_CODE" ]; then
        rm -f FIRMWARE_CODE
        rm -rf vendor
        for fw in \
            audience-manta-lrx21p-d3749cac.tgz \
            broadcom-manta-lrx21p-23ffa32a.tgz \
            samsung-manta-lrx21p-0bbebdb4.tgz; do
            [ -e "$fw" ] || {
                echo "Downloading proprietary firmware: $fw"
                autoex="extract-"$(echo "$fw" | cut -d- -f -2)".sh"
                rm -f "$autoex"
                curl "$AOSP_BIN_URL"/"$fw" | tar xzf -
                echo "Running installer: $autoex"
                ./"$autoex"
            }
        done
        echo "$AOSP_FW_CODE" > FIRMWARE_CODE
    fi
    echo "Proprietary firmware code: $(cat FIRMWARE_CODE)"

    repo sync $(jopt)

    cd - > /dev/null
    echo "Replacing Mali GPU driver libGLES_mali.so"
    rm -f aosp/vendor/samsung/manta/proprietary/libGLES_mali.so
    tar xf \
        "$mali_archive" \
        -C aosp/vendor/samsung/manta/proprietary/ \
        --strip=1 \
        "$mali_name"/libGLES_mali.so
}

cmd_get_kernel()
{
    arg_url="${1-$KERNEL_URL}"

    # Create initial git repository if not already present
    [ -d kernel ] || git clone "$arg_url" kernel

    cd kernel

    # If not already there, create branch with given revision and apply patches
    git branch | grep "$MALI_VERSION" > /dev/null || {
        git checkout "$KERNEL_BRANCH"

        echo "Checking out revision $KERNEL_REV into branch $MALI_VERSION"
        git checkout "$KERNEL_REV" -b "$MALI_VERSION"

        patch_dir="../data/patch-$MALI_VERSION"
        [ -d "$patch_dir" ] && {
            echo "Applying patches from $patch_dir"
            git am "$patch_dir"/*
        }
    }

    ARCH=arm make manta_defconfig

    cd - > /dev/null
}

cmd_build_kernel()
{
    cd kernel
    make $(jopt)
    cd - > /dev/null
}

cmd_build_gator()
{
    if [ -d gator-driver ]; then
        echo "Building Gator driver"
        cd gator-driver
        GATOR_WITH_MALI_SUPPORT=MALI_MIDGARD \
            make $(jopt) -C ../kernel M="$PWD" modules
        cd - > /dev/null
    else
        echo "No Gator driver source code found."
    fi

    if [ -d gator-daemon ]; then
        echo "Building Gator daemon"
        [ -d gator-daemon/jni ] || {
            mv gator-daemon gator-daemon-jni
            mkdir gator-daemon
            mv gator-daemon-jni gator-daemon/jni
        }
        cd gator-daemon
        ndk-build $(jopt)
        cd - > /dev/null
    else
        echo "No Gator daemon source code found."
    fi
}

cmd_build_aosp()
{
    cp kernel/arch/arm/boot/zImage aosp/device/samsung/manta/kernel
    cd aosp
    . build/envsetup.sh
    lunch aosp_manta-eng
    rm -rf out/target/product/manta/system
    rm -rf out/target/product/manta/root
    rm -f out/target/product/manta/*.img
    rm -f out/target/product/manta/obj/lib/libGLES_mali.so
    sysbin="out/target/product/manta/system/bin"
    mkdir -p "$sysbin"
    gator_driver="../gator-driver/gator.ko"
    [ -e "$gator_driver" ] && install -m 0644 "$gator_driver" "$sysbin"
    gator_daemon="../gator-daemon/libs/armeabi-v7a/gatord"
    [ -e "$gator_daemon" ] && install -m 0755 "$gator_daemon" "$sysbin"
    make $(jopt)
    cd - > /dev/null
}

cmd_package()
{
    [ -d "$AOSP_FACTORY" ] || {
        echo "Downloading factory binaries: $AOSP_FACTORY_TGZ"
        curl "$AOSP_BIN_URL"/"$AOSP_FACTORY_TGZ" | tar xzf -
    }

    pkg="$FULL_PACKAGE"
    rm -f "$pkg".tar.gz
    rm -rf "$pkg"
    mkdir -p "$pkg"
    zip -j "$pkg"/image-"$pkg".zip \
        data/android-info.txt \
        aosp/out/target/product/manta/{boot,cache,recovery,system,userdata}.img
    cp "$AOSP_FACTORY"/flash* "$pkg"/
    sed -i "s/$AOSP_FACTORY/$pkg/g" "$pkg"/flash-all.{sh,bat}
    cp "$AOSP_FACTORY"/bootloader-manta-mantamf01.img "$pkg"/

    echo ""
    echo "Packaged output:"
    ls -l "$pkg"/*

    echo ""
    echo "To flash your device, reboot it in bootloader mode by pressing"
    echo "Volume-Up + Volume-Down + Power or running this command:"
    echo "  adb reboot bootloader"
    echo "Then run:"
    echo "  cd $pkg"
    echo "  sudo ./nexus10-setup.sh flashall"
}

cmd_do_everything()
{
    cmd_get_aosp
    cmd_get_kernel
    cmd_build_kernel
    cmd_build_gator
    cmd_build_aosp
    cmd_package
}

# Run the command if it's valid, otherwise abort
type cmd_$cmd > /dev/null 2>&1 || print_usage_exit
cmd_$cmd $@

exit 0
